#! /usr/bin/perl -w
#
###############################################################################
#
# Copyright (C) 2004-2009 by Kostis Netzwerkberatung
# Talstr. 25, D-63322 Roedermark, Germany
# kosta@kostis.de, http://www.kostis.de/
#
# 2009-02-03: V1.40 KK
#     - initial re-release
#
# You may use this software free of charge at your own risk.
#
###############################################################################
#
# to be included by "CiscoTools" report scripts
#
###############################################################################

use CGI qw/:standard/ ;	# for url_param()
use File::Basename ;	# for basename(), dirname ()
use Socket ;		# for gethostbyaddr ()

###############################################################################
# global hashes
###############################################################################

###############################################################################
# result from read_oui ()
###############################################################################

%oui = () ;		# from http://standards.ieee.org/regauth/oui/oui.txt

###############################################################################
# result from read_arp ()
###############################################################################

%arplog = () ;		# map "mac,ipaddr,vlan" to "isodate"

###############################################################################
# result from map_arp ()
###############################################################################

%mac2ipaddr = () ;	# map "mac" to "ipaddr[,ipaddr...]"
%ipaddr2mac = () ;	# map "ipaddr" to "mac[,mac...]"
%ipaddr2vlan = () ;	# map "ipaddr" to "vlan[,vlan...]"
%ipaddr2macvlan = () ;	# map "ipaddr" to "mac;vlan[,mac;vlan...]"

###############################################################################
# result from map_dns ()
###############################################################################

%revdns = () ;		# map "ipaddr" to "hostname" (not FQDN)

###############################################################################
# result from read_mac ()
###############################################################################

%maclog = () ;		# map "mac,switch,interface,vlan" to "isodate"

###############################################################################
# result from map_mac () ;
###############################################################################

%interface2mac = () ;	# map "switch,interface,vlan" to "mac[,mac...]"
%interface2vlan = () ;	# map "switch,interface" to "vlan[,vlan...]"
%mac2interface = () ;	# map "mac" to "switch,interface[;switch,interface...]"
%mac2vlan = () ;	# map "mac" to "vlan[,vlan...]"

###############################################################################
# result from read_portmaps ()
###############################################################################

%portmaps = () ;	# map "switch,interface" to "a_switch,a_interface"

###############################################################################
# result from read_ignorearp ()
###############################################################################

%ignorearp = () ;	# hash of ipaddr block to be ignored for some reports

###############################################################################
# result from read_ignoremac ()
###############################################################################

%ignoremac = () ;	# hash of mac addresses to be ignored for some reports

###############################################################################
# result from read_ipblock ()
###############################################################################

%ipblocks = () ;	# hash of ipblocks and descriptions

###############################################################################
# result from read_locations ()
###############################################################################

%locations = () ;	# hash of locations and descriptions
%plocations = () ;	# hash of permitted locations

###############################################################################
# result from count_vlans ()
###############################################################################

%countvlans = () ;	# map "vlan" to "# of interfaces in vlan"

###############################################################################
# variables read from http server
###############################################################################

$username = "" ;
if (exists ($ENV{'REMOTE_USER'}))
{
	$username = $ENV{'REMOTE_USER'} ;
}

###############################################################################
# variables read from parameters
###############################################################################

$location      = url_param ('location') ;
$findipaddr    = url_param ('findipaddr') ;
$findipblock   = url_param ('findipblock') ;
$findmac       = url_param ('findmac') ;
$findswitch    = url_param ('findswitch') ;
$findinterface = url_param ('findinterface') ;
$findvendor    = url_param ('findvendor') ;
$listafter     = url_param ('listafter') ;
$listbefore    = url_param ('listbefore') ;
$listnodns     = url_param ('listnodns') ;
$do_ignorearp  = url_param ('ignorearp') ;
$do_ignoremac  = url_param ('ignoremac') ;

# set default for ignorearp
if ("$do_ignorearp" eq "")
{
	$do_ignorearp = "yes" ;
}

# set default for ignoremac
if ("$do_ignoremac" eq "")
{
	$do_ignoremac = "yes" ;
}

$listdate = "" ;
$url_listdate = "" ;
if ("$listafter" ne "")
{
	$listdate = "after $listafter" ;
	$url_listdate = "&amp;listafter=$listafter" ;
}
elsif ("$listbefore" ne "")
{
	$listdate = "before $listbefore" ;
	$url_listdate = "&amp;listbefore=$listbefore" ;
}
$listafter =~ s/_/ / ;
$listbefore =~ s/_/ / ;
$listdate =~ s/_/ / ;

###############################################################################
# location of Cisco Tools data
###############################################################################

$HTMLDIR    = "/srv/www/htdocs/cisco/data/$location" ;
$MAC        = "$HTMLDIR/maclog.txt" ;	# maglog data filename with full path
$ARP        = "$HTMLDIR/arplog.txt" ;	# arplog data filename with full path

$CGIDIR     = dirname ($0) ;		# CiscoTools cgi-bin directory
$CGIDAT     = $CGIDIR . "/$location" ;	# script data directory for location
$OUI        = "$CGIDIR/oui.txt" ;	# oui data file with full path
$LOCATIONS  = "$CGIDIR/locations.txt" ;	# locations data file with full path
$PLOCATIONS = "$CGIDIR/perms.txt" ;	# permitted locations (per user)

$PORTMAPS   = "$CGIDAT/portmaps.txt" ;	# portmaps data file with full path
$IGNOREMAC  = "$CGIDAT/ignoremac.txt" ;	# ignoremac data file with full path
$IGNOREARP  = "$CGIDAT/ignorearp.txt" ;	# ignorearp data file with full path
$IPBLOCKS   = "$CGIDAT/ipblocks.txt" ;	# ipblocks data file with full path

###############################################################################
#
# sub CiscoTools_info (infotype)
#
# return library information
#
# returns:
#
# key is "infotype", value is "infovalue"
#
###############################################################################

sub CiscoTools_info ($)
{
	my $infotype = shift ;

	my %CiscoToolsInfo =
	(
		"copyright" => "Copyright (C) 2004-2009 by Kostis Netzwerkberatung",
		"author"    => "Konstantinos Kostis",
		"address"   => "Talstr. 25, D-63322 Roedermark, Germany",
		"internet"  => "kosta\@kostis.de, http://www.kostis.de/",
		"license"   => "You may use this software free of charge at your own risk.",
		"version"   => "V1.40 (2009-02-03)"
	) ;

	return ($CiscoToolsInfo{$infotype}) ;
}

###############################################################################
#
# sub iso8601_now (what)
#
# return current date/time in ISO8601 format
#
# returns:
#
# what=full	CCYY-MM-DD_hh:mm:ss 
# what=date	CCYY-MM-DD
# what=time	hh:mm:ss 
# what=midnight	CCYY-MM-DD_00:00:00 
#
###############################################################################

sub iso8601_now ($)
{
	my $what = shift ;

	(my $s, $m, $h, $D, $M, $Y, $wd, $yd, $id) = localtime (time ()) ;

	my $date = sprintf ("%04d-%02d-%02d", $Y + 1900, $M + 1, $D) ;
	my $time = sprintf ("%02d:%02d:%02d", $h, $m, $s) ;
	my $full = $date . "_" . $time ;
	my $midnight = $date . "_00:00:00" ;

	if ("$what" eq "date")
	{
		return ($date) ;
	}
	elsif ("$what" eq "time")
	{
		return ($time) ;
	}
	elsif ("$what" eq "full")
	{
		return ($full) ;
	}
	elsif ("$what" eq "midnight")
	{
		return ($midnight) ;
	}

	return ("") ;
}

###############################################################################
#
# sub add_element (list, element, delim)
#
# add element to a list (prevent duplicates)
#
# returns:
#
# list with element added (if already in list, not added again)
#
###############################################################################

sub add_element ($$$)
{
	my $list = shift ;
	my $element = shift ;
	my $delim = shift ;
	my @array = () ;
	my %hash = () ;

	#######################################################################
	# don't try to add an empty element
	#######################################################################

	if ("$element" eq "")
	{
		return ($list) ;
	}

	#######################################################################
	# create hash of elements in list only if list is not empty
	#######################################################################

	if ("$list" ne "")
	{
		@array = split ($delim, $list) ;
		foreach my $i (sort @array)
		{
			$hash { $i } = 1 ;
		}
	}

	#######################################################################
	# don't add an element more than once
	#######################################################################

	if (not exists ($hash {$element}))
	{
		$list = $list . $delim . $element ;
	}

	return ($list) ;
}

###############################################################################
#
# sub ipblock (ipaddr)
#
# return first three octets from a given ipaddr
#
# returns:
#
# ipblock
#
###############################################################################

sub ipblock ($)
{
	my $ipaddr = shift ;

	$ipaddr =~ s/\.\d+$// ;

	return ($ipaddr) ;
}

###############################################################################
#
# sub mac2vendor (mac)
#
# return vendor associated with mac (if possible)
#
# read_oui () must be called before
#
# returns:
#
# vendor
#
###############################################################################

sub mac2vendor ($)
{
	my $mac = shift ;

	my $vendor = "" ;
	my $moui = substr ($mac, 0, 8) ;
	if (exists ($oui {$moui}))
	{
		$vendor = $oui {$moui} ;
	}

	return ($vendor) ;
}

###############################################################################
#
# sub read_oui ()
#
# read vendor information (from list of vendors IEEE OUI)
#
# returns:
#
# number of entries read
#
# changes/fills:
#
# %oui			key is "XX-XX-XX", value is "Vendor String"
#
###############################################################################

sub read_oui ()
{
	%oui = () ;

	my $count = 0 ;

	#######################################################################
	# check if oui data file is present
	#######################################################################

	if (not -e $OUI)
	{
		return (0) ;
	}

	#######################################################################
	# open list of OUI (IEEE)
	#######################################################################

	open (F, "$OUI") ;
	while (<F>)
	{
		chomp () ;

		###############################################################
		# extract vendor information from list of OUI (IEEE)
		###############################################################

		if (/(\w\w-\w\w-\w\w)\s+\(hex\)\s+(.+)$/)
		{
			$oui {"$1"} = "$2" ;
			$count++ ;
		}
	}
	close (F) ;

	return ($count) ;
}

###############################################################################
#
# sub read_portmaps ()
#
# read portmaps
#
# returns:
#
# number of entries read
#
# changes/fills:
#
# %portmaps		key is "$switch,$interface", value is "adjacent"
#
###############################################################################

sub read_portmaps ()
{
        my $switch = "" ;
        my $interface = "" ;
        my $a_switch = "" ;
        my $a_interface = "" ;
        my $key = "" ;

	%portmaps = () ;

	my $count = 0 ;

	#######################################################################
	# check if portmaps data file is present
	#######################################################################

	if (not -e $PORTMAPS)
	{
		return (0) ;
	}

	#######################################################################
	# read portmap data
	#######################################################################

        open (F, $PORTMAPS) ;
        while (<F>)
        {
                chomp () ;

                ###############################################################
                # ignore empty lines and comment lines
                # valid lines always begin with a letter or digit
                ###############################################################

                if (/^\w/)
                {
                        ($switch, $interface, $a_switch, $a_interface) = split (",") ;
                        $key = "$switch,$interface" ;
                        $portmaps {"$key"} = "$a_switch" . "," . "$a_interface" ;
			$count++ ;
                }
        }
        close (F) ;

	return ($count) ;
}

###############################################################################
#
# read_ignorearp ()
#
# read list of ipblocks to be ignored
#
# read_arp () must be called before
# calls to map_arp () must be made after calling this (if ignorearp is wanted)
#
# returns:
#
# number of entries read
#
# changes/fills:
#
# %ignorearp		key is "ipblock", value is "comment" or ""
#
###############################################################################

sub read_ignorearp ()
{
	my $ipblock = "" ;
	my $comment = "" ;
	my $dummy = "" ;

	%ignorearp = () ;

	my $count = 0 ;

	#######################################################################
	# check if ignorearp data file is present
	#######################################################################

	if (not -e $IGNOREARP)
	{
		return (0) ;
	}

	#######################################################################
	# read ignorearp data
	#######################################################################

        open (F, $IGNOREARP) ;
        while (<F>)
        {
                chomp () ;
                if (/^(\d+\.\d+\.\d+),(.+)/)
                {
                        $ipblock = $1 ;
			$comment = $2 ;
                        $ignorearp {$ipblock} = "$comment" ;
			$count++ ;
                }
        }
        close (F) ;

	#######################################################################
	# clean up arplog
	#######################################################################

	foreach $key (sort keys %arplog)
	{
		(my $mac, my $ipaddr) = split (",", $key) ;

		###############################################################
		# handle complete ipaddr
		###############################################################

		if (exists ($ignorearp {ipblock ($ipaddr)}))
		{
			delete ($arplog {$key}) ;
		}
	}

	return ($count) ;
}

###############################################################################
#
# read_ignoremac ()
#
# read list of mac addresses to be ignored
#
# read_mac () must be called before
# calls to map_mac () must be made after calling this (if ignoremac is wanted)
#
# returns:
#
# number of entries read
#
# changes/fills:
#
# %ignoremac		key is "mac", value is "comment" or ""
#
###############################################################################

sub read_ignoremac ()
{
	my $mac = "" ;
	my $comment = "" ;
	my $switch = "" ;
	my $interface = "" ;
	my $vlan = "" ;
	my $ipaddr = "" ;

	%ignoremac = () ;

	#######################################################################
	# check if ignoremac data file is present
	#######################################################################

	if (not -e $IGNOREMAC)
	{
		return (0) ;
	}

	#######################################################################
	# read ignoremac data
	#######################################################################

        open (F, $IGNOREMAC) ;
        while (<F>)
        {
                chomp () ;

		###############################################################
		# allow ignoring from single macs up to complete vendors
		#
		# XX-XX-XX-XX-XX-XX	ignore single mac
		# XX-XX-XX-XX-XX	ignore up to 256 macs
		# XX-XX-XX-XX		ignore up to 65536 macs
		# XX-XX-XX		ignore complete vendor oui
		###############################################################

                if (/^\w\w(-\w\w){2,5},/)
                {
			($mac, $comment) = split (",") ;
			$mac = uc ($mac) ;
                        $ignoremac {"$mac"} = "$comment" ;
                }
        }
        close (F) ;

	#######################################################################
	# clean up arplog
	#######################################################################

	foreach $key (sort keys %arplog)
	{
		($mac, $ipaddr, $vlan) = split (",", $key) ;

		###############################################################
		# start with single mac, end with vendor oui
		###############################################################

		my $doit = "no" ;
		for (my $mlen = 17 ; $mlen >= 8 ; $mlen -= 3)
		{
			$mac = substr ($mac, 0, $mlen) ;
			if (exists ($ignoremac {$mac}))
			{
				$doit = "yes" ;
			}
		}

		###############################################################
		# use the mechanism to prevent multiple deletion attempts
		###############################################################

		if ("$doit" eq "yes")
		{
			delete ($arplog {$key}) ;
		}
	}

	#######################################################################
	# clean up maclog
	#######################################################################

	my $n_delmac = 0 ;
	foreach $key (sort keys %maclog)
	{
		($mac, $switch, $interface, $vlan) = split (",", $key) ;

		###############################################################
		# start with single mac, end with vendor oui
		###############################################################

		my $doit = "no" ;
		for (my $mlen = 17 ; $mlen >= 8 ; $mlen -= 3)
		{
			$mac = substr ($mac, 0, $mlen) ;
			if (exists ($ignoremac {$mac}))
			{
				$doit = "yes" ;
			}
		}

		###############################################################
		# use the mechanism to prevent multiple deletion attempts
		###############################################################

		if ("$doit" eq "yes")
		{
			delete ($maclog {$key}) ;
			$n_delmac++ ;
		}
	}

	return ("$n_delmac") ;
}

###############################################################################
#
# read_ipblocks ()
#
# read list of ipblocks descriptions
#
# returns:
#
#  number of entries read
#
# changes/fills:
#
# %ipblocks		key is "ipblock", value is "description"
#
###############################################################################

sub read_ipblocks ()
{
	my $ipblock = "" ;
	my $description = "" ;

	%ipblocks = () ;

	my $count ;

	#######################################################################
	# check if ipblocks data file is present
	#######################################################################

	if (not -e $IPBLOCKS)
	{
		return (0) ;
	}

	#######################################################################
	# read ipblocks data
	#######################################################################

	$count = 0 ;
        open (F, $IPBLOCKS) ;
        while (<F>)
        {
                chomp () ;
                if (/^(\d+\.\d+\.\d+),(.+)$/)
                {
			$ipblock = "$1" ;
			$description = "$2" ;
                        $ipblocks {"$ipblock"} = "$description" ;
			$count++ ;
                }
        }
        close (F) ;

	return ($count) ;
}

###############################################################################
#
# read_locations ()
#
# read list of location descriptions
#
# returns:
#
#  number of entries read
#
# changes/fills:
#
# %locations		key is "location", value is "description"
# %plocations		key is "user", value is "location[,location]"
#
###############################################################################

sub read_locations ()
{
	my $location = "" ;
	my $description = "" ;

	%locations = () ;
	%plocations = () ;

	my $count ;

	#######################################################################
	# check if locations data file is present
	#######################################################################

	if (not -e $LOCATIONS)
	{
		return (0) ;
	}

	#######################################################################
	# read locations data
	#######################################################################

	$count = 0 ;
        open (L, $LOCATIONS) ;
        while (<L>)
        {
                chomp () ;
		my $line = "$_" ;
		if (("$line" ne "") && (substr ($line, 0, 1) ne "#"))
		{
			(my $location, my $description) = split (",", $line) ;
			$locations {$location} = $description ;
			$count++ ;
		}
	}
	close (L) ;

	#######################################################################
	# check if locations data file is present
	#######################################################################

	if (not -e $PLOCATIONS)
	{
		return ($count) ;
	}

	#######################################################################
	# read permitted locations
	#######################################################################

        open (L, $PLOCATIONS) ;
        while (<L>)
        {
                chomp () ;
		my $line = "$_" ;
		if (("$line" ne "") && (substr ($line, 0, 1) ne "#"))
		{
			(my $uname, my $ploc) = split (";", $line) ;
			if ("$uname" eq "$username")
			{
				if ("$ploc" eq "*")
				{
					foreach my $location (keys %locations)
					{
						$plocations {$location} = 1 ;
					}
				}
				else
				{
					my @perms = split (",", $ploc) ;
					foreach my $location (sort @perms)
					{
						$plocations {$location} = 1 ;
					}
				}
				last ;
			}
		}
	}
	close (L) ;

	return ($count) ;
}

###############################################################################
#
# sub ipaddr2hostname (ipaddr)
#
# return hostname using reverse DNS cache only
#
###############################################################################

sub ipaddr2hostname ($)
{
	my $ipaddr = shift ;

	my $hostname = "" ;
	if (exists ($revdns {$ipaddr}))
	{
		$hostname = $revdns {$ipaddr} ;
	}

	return ($hostname) ;
}

###############################################################################
#
# sub host (ipaddr)
#
# return hostname using reverse DNS lookup, remove domain name (no FQDN)
#
###############################################################################

sub host ($)
{
	my $ipaddr = shift ;

	#######################################################################
	# translate ipaddr to hostname
	#######################################################################

	my $hostname = gethostbyaddr (inet_aton ($ipaddr), AF_INET) ;

	#######################################################################
	# if hostname was returned remove domain name from hostname
	#######################################################################

	if ("$hostname" ne "")
	{
		###############################################################
		# remove domain name (retrieve hostname)
		###############################################################

		my $i = index ($hostname, ".") ;
		if ($i > 0)
		{
			$hostname = substr ($hostname, 0, $i) ;
		}
	}

	return ($hostname) ;
}

###############################################################################
#
# sub read_arp ()
#
# read arp data (mac, ipaddr, vlan, date, time)
#
# returns:
#
# number of entries read
#
# changes/fills:
#
# %arplog		key is "mac,ipaddr,vlan", value is "isodate"
#
###############################################################################

sub read_arp ()
{
	%arplog = () ;

	#######################################################################
	# check if arplog data file is present
	#######################################################################

	if (not -e $ARP)
	{
		return (0) ;
	}

	#######################################################################
	# open arp data file (mac,ipaddr;isodate)
	#######################################################################

	my $count = 0 ;
	open (F, "$ARP") ;
	while (<F>)
	{
		chomp () ;

		###############################################################
		# split key and value and store raw arplog data
		###############################################################

		(my $key, my $isodate) = split (";") ;
		(my $mac, my $ipaddr, my $vlan) = split (",", $key) ;
		if ("$vlan" eq "")
		{
			$key = $key . "n/a" ;
		}
		if ("$mac" ne "FF-FF-FF-FF-FF-FF")
		{
			$arplog {$key} = $isodate ;
			$count++ ;
		}
	}
	close (F) ;

	return ($count) ;
}

###############################################################################
#
# sub map_arp ()
#
# map mac to ipaddr and ipaddr to mac
#
# returns:
#
# number of arp entries processed
#
# changes/fills:
#
# %mac2ipaddr		key is "mac", value is "ipaddr[,ipaddr]"
# %ipaddr2mac		key is "ipaddr", value is "mac[,mac]"
# %ipaddr2vlan		key is "ipaddr", value is "vlan[,vlan]"
# %ipaddr2macvlan	key is "ipaddr", value is "mac;vlan[,mac;vlan]"
#
###############################################################################

sub map_arp ()
{
	%mac2ipaddr = () ;
	%ipaddr2mac = () ;
	%ipaddr2vlan = () ;
	%ipaddr2macvlan = () ;

	my $count = 0 ;
	foreach my $key (keys %arplog)
	{
		###############################################################
		# get mac, ipaddr and vlan from key
		###############################################################

		(my $mac, my $ipaddr, my $vlan) = split (",", $key) ;
		my $macvlan = $mac . ";" . $vlan ;

		###############################################################
		# map mac to ipaddr
		###############################################################

		if (not exists ($mac2ipaddr {$mac}))
		{
			$mac2ipaddr {$mac} = $ipaddr ;
		}
		else
		{
			$mac2ipaddr {$mac} = add_element ($mac2ipaddr {$mac}, $ipaddr, ",") ;
		}

		###############################################################
		# map ipaddr to mac
		###############################################################

		if (not exists ($ipaddr2mac {$ipaddr}))
		{
			$ipaddr2mac {$ipaddr} = $mac ;
		}
		else
		{
			$ipaddr2mac {$ipaddr} = add_element ($ipaddr2mac {$ipaddr}, $mac, ",") ;
		}

		###############################################################
		# map ipaddr to vlan
		###############################################################

		if (not exists ($ipaddr2vlan {$ipaddr}))
		{
			$ipaddr2vlan {$ipaddr} = $vlan ;
		}
		else
		{
			$ipaddr2vlan {$ipaddr} = add_element ($ipaddr2vlan {$ipaddr}, $vlan, ",") ;
		}

		if (not exists ($ipaddr2macvlan {$ipaddr}))
		{
			$ipaddr2macvlan {$ipaddr} = $macvlan ;
		}
		else
		{
			$ipaddr2macvlan {$ipaddr} = add_element ($ipaddr2macvlan {$ipaddr}, $macvlan, ",") ;
		}

		###############################################################
		# map ipaddr to mac;vlan
		###############################################################


		$count++ ;
	}

	return ($count) ;
}

###############################################################################
#
# sub map_dns ()
#
# resolve hostnames for known ipaddr
#
# returns:
#
# number of resolved hostnames
#
# changes/fills:
#
# %revdns	key is "ipaddr", value is "hostname"
#
###############################################################################

sub map_dns ()
{
	%revdns = () ;

	my $count = 0 ;
	foreach my $ipaddr (keys %ipaddr2mac)
	{
		###############################################################
		# lookup hostname using ipaddr then store information
		# avoid looking up the same ipaddr more than once
		###############################################################

		if (not exists ($revdns {$ipaddr}))
		{
			my $hostname = host ($ipaddr) ;
			if ("$hostname" ne "")
			{
				$revdns {$ipaddr} = $hostname ;
				$count++ ;
			}
		}
	}

	return ($count) ;
}

###############################################################################
#
# sub read_mac ()
#
# read mac-address-table data (mac, switch, interface, vlan, date, time)
#
# returns:
#
# number of entries read
#
# changes/fills:
#
# %maclog	key is "mac,switch,interface,vlan", value is "isodate"
#
###############################################################################

sub read_mac ()
{
	%maclog = () ;

	#######################################################################
	# check if arplog data file is present
	#######################################################################

	if (not -e $MAC)
	{
		return (0) ;
	}

	#######################################################################
	# open mac data file (mac,switch,interface;isodate)
	#######################################################################

	my $count = 0 ;
	open (F, "$MAC") ;
	while (<F>)
	{
		chomp () ;

		###############################################################
		# split key and value and store maclog
		###############################################################

		(my $key, my $isodate) = split (";") ;
		(my $mac, my $switch, my $vlan) = split (",", $key) ;
		if ("$vlan" eq "")
		{
			$key = $key . "n/a" ;
		}
		if ("$mac" ne "FF-FF-FF-FF-FF-FF")
		{
			$maclog {$key} = $isodate ;
			$count++ ;
		}
	}
	close (F) ;

	return ($count) ;
}

###############################################################################
#
# sub map_mac ()
#
# map maclog data per interface
#
# returns:
#
# nothing
#
# changes/fills:
#
# %interface2mac	key is "switch,interface,vlan", value is "mac[,mac...]"
# %interface2vlan	key is "switch,interface", value is "vlan[,vlan...]"
# %mac2interface	key is "mac", value is "switch,interface,vlan[;...]"
# %mac2vlan		key is "mac", value is "vlan[,vlan...]"
#
###############################################################################

sub map_mac ()
{
	%interface2mac = () ;
	%interface2vlan = () ;
	%mac2interface = () ;
	%mac2vlan = () ;

	foreach my $key (sort keys %maclog)
	{
		(my $mac, my $switch, my $interface, my $vlan) = split (",", $key) ;
		my $mmap = "$switch,$interface,$vlan" ;
		my $vmap = "$switch,$interface" ;

		if (not exists ($interface2mac {"$mmap"}))
		{
			$interface2mac {"$mmap"} = "$mac" ;
		}
		else
		{
			$interface2mac {"$mmap"} = add_element ($interface2mac {$mmap}, $mac, ",") ;
		}

		if (not exists ($interface2vlan {"$vmap"}))
		{
			$interface2vlan {"$vmap"} = "$vlan" ;
		}
		else
		{
			$interface2vlan {"$vmap"} = add_element ($interface2vlan {$vmap}, $vlan, ",") ;
		}

		if (not exists ($mac2interface {"$mac"}))
		{
			$mac2interface {"$mac"} = "$mmap" ;
		}
		else
		{
			$mac2interface {"$mac"} = add_element ($mac2interface {$mac}, $mmap, ";") ;
		}

		if (not exists ($mac2vlan {"$mac"}))
		{
			$mac2vlan {"$mac"} = "$vlan" ;
		}
		else
		{
			$mac2vlan {"$mac"} = add_element ($mac2vlan {$mac}, $vlan, ",") ;
		}
	}
}

###############################################################################
#
# sub count_vlans ()
#
# count maclog data per vlan
#
# returns:
#
# number of vlans
#
# changes/fills:
#
# %countvlans		key is "vlan", value is "# of interfaces in vlan"
#
###############################################################################

sub count_vlans ()
{
	%countvlans = () ;

	my $cvlan = 0 ;
	foreach my $key (sort keys %maclog)
	{
		(my $mac, my $switch, my $interface, my $vlan) = split (",", $key) ;
		if (not exists ($countvlans {$vlan}))
		{
			$cvlan++ ;
			$countvlans {$vlan} = 1 ;
		}
		else
		{
			++$countvlans {$vlan} ;
		}
	}

	return ($cvlan) ;
}

###############################################################################
# the last line is required by require
###############################################################################

1 ;
