#! /usr/bin/perl
#
###############################################################################
#
# Copyright (C) 2008-2009 by Kostis Netzwerkberatung
# Talstr. 25, D-63322 Roedermark, Germany
# kosta@kostis.de, http://www.kostis.de/
#
# 2009-02-03: V1.40 KK
#     - re-release
#
# You may use this software free of charge at your own risk.
#
###############################################################################
#
# display arplog data in HTML table format resolving hostname (sort by ipaddr)
#
###############################################################################

require "CiscoTools.pl" ;

###############################################################################

my $version = "1.40 (2009-02-03)" ;
my $prgnam = basename ($0) ;

###############################################################################
# HTML title
###############################################################################

$TITLE="CiscoTools $prgnam $location" ;
$H1="<strong><font color=\"#0000A0\">Cisco</font><font color=\"#0000FF\">Tools</font></strong> $prgnam $location" ;

###############################################################################

sub getvlan ($$)
{
	my $mac = shift ;
	my $ipaddr = shift ;

	if (exists ($ipaddr2vlan {$ipaddr}))
	{
		my $vlan_list = $ipaddr2vlan {$ipaddr} ;
		my @a_vlans = split (",", $vlan_list) ;
		foreach my $vlan (sort @a_vlans)
		{
			my $key = $mac . "," . $ipaddr . "," . $vlan ;
			if (exists ($arplog {$key}))
			{
				return ($vlan) ;
			}
		}
	}

	return ("") ;
}

###############################################################################
#
# main ()
#
###############################################################################

###############################################################################
# read data
###############################################################################

my $num_oui = read_oui () ;	# read OUI data (from IEEE)
my $num_arp = read_arp () ;	# read arplog data (show arp)

###############################################################################
# remove unwanted data
###############################################################################

my $num_ignorearp = 0 ;
if ("$do_ignorearp" eq "yes")
{
	$num_ignorearp = read_ignorearp () ;
}

my $num_ignoremac = 0 ;
if ("$do_ignoremac" eq "yes")
{
	$num_ignoremac = read_ignoremac () ;
}

###############################################################################
# map data
###############################################################################

my $num_arp = map_arp () ;	# map mac to ipaddr and vice versa
my $num_dns = map_dns () ;	# map ipaddr to hostname

###############################################################################
# display HTML header
###############################################################################

print <<HTML_HEAD;
Content-type: text/html

<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN">

<html>

  <head>

    <title>$TITLE</title>
    <meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1">
    <meta http-equiv=Content-Language content=en>
    <meta content="CiscoTools" name=description>
    <meta content="Kostis Netzwerkberatung" name=author>
    <meta content="Kostis Netzwerkberatung" name=publisher>
    <meta content="Kostis Netzwerkberatung" name=copyright>
    <meta name="GENERATOR" content="$prgnam $version">
    <link rev="made" href="mailto:webmaster@kostis.de">
    <link href="/cisco/style.css" rel=stylesheet>

  </head>

  <body>

    <h1>$H1</h1>

HTML_HEAD

###############################################################################
# check if user is allowed to view this location
###############################################################################

read_locations () ;

if (not exists ($plocations {$location}))
{
###############################################################################
	print <<HTML_NOPERM;
    <p>Your account $username has no permission to view location $location</p>

    <p>$prgnam $version</p>

  </body>

</html>
HTML_NOPERM
###############################################################################
	exit ;
}

###############################################################################
# if an ipblock is given get additional information
###############################################################################

if ("$findipblock" ne "")
{
	read_ipblocks () ;
	my $description = "" ;
	if (exists ($ipblocks {$findipblock}))
	{
		$description = " (" . $ipblocks {$findipblock} . ")" ;
	}
	print <<HTML_IPBLOCK;
    <h2>ipblock $findipblock$description</h2>

HTML_IPBLOCK
}

###############################################################################
# if an ipblock is given get additional information
###############################################################################

if ("$listdate" ne "")
{
	print <<HTML_LISTDATE;

    <h2>display limited to data $listdate</h2>

HTML_LISTDATE
}

###############################################################################
# if listnodns=yes is given say so
###############################################################################

if ("$listnodns" eq "yes")
{
	print <<HTML_LISTNODNS;

    <h2>display limited to ipaddr without reverse dns entries</h2>

HTML_LISTNODNS
}

print <<HTML_TABLE;
    <table>

HTML_TABLE

###############################################################################
# walk through data using ipaddr as sort key
###############################################################################

my $last_ipblock = "" ;
my $num_listed = 0 ;
foreach my $ipaddr (sort { ($hc{$a} ||= inet_aton($a)) cmp ($hc{$b} ||= inet_aton($b)) } keys %ipaddr2mac)
{ 
	# an ipaddr may be mapped to a number of macs
	my @mactab = split (",", $ipaddr2mac {$ipaddr}) ;

	# walk through all macs for a given ipaddr
	foreach my $mac (sort @mactab)
	{
		###############################################################
		# determine vendor
		###############################################################

		my $vendor = mac2vendor ($mac) ;

		###############################################################
		# determine hostname
		###############################################################

		my $hostname = ipaddr2hostname ($ipaddr) ;

		###############################################################
		# find vlan associated to mac,ipaddr
		###############################################################

		my $key = "" ;
		my $vlan = getvlan ($mac, $ipaddr) ;
		if ("$vlan" ne "")
		{
			$key = $mac . "," . $ipaddr . "," . $vlan ;
		}

		###############################################################
		# determine last seen date and time
		###############################################################

		if ("$key" ne "")
		{
			my $ldate = $arplog {$key} ;
			$ldate =~ s/_/ / ;
			(my $date, my $time) = split (" ", $ldate) ;
			my $b_ipblock = ipblock ($ipaddr) ;

			my $doit = "no" ;
			if ("$findipblock" eq "")
			{
				$doit = "yes" ;
			}
			else
			{
				if ("$findipblock" eq "$b_ipblock")
				{
					$doit = "yes" ;
				}
			}

			if (("$listafter" ne "") && ("$ldate" lt "$listafter"))
			{
				$doit = "no" ;
			}
			elsif (("$listbefore" ne "") && ("$ldate" gt "$listbefore"))
			{
				$doit = "no" ;
			}

			if (("$listnodns" eq "yes") && ("$hostname" ne ""))
			{
				$doit = "no" ;
			}

			if ("$doit" eq "yes")
			{

###############################################################################
# display table header
###############################################################################
				if ("$last_ipblock" ne "$b_ipblock")
				{
					print <<HTML_TABLE_HEAD;
      <tr>
        <td class="title">ipaddr&nbsp;</td>
        <td class="title">mac&nbsp;</td>
        <td class="title">hostname&nbsp;&nbsp;</td>
        <td class="title">vlan&nbsp;</td>
        <td class="title">last&nbsp;seen&nbsp;</td>
        <td class="title">vendor&nbsp;</td>
     </tr>

HTML_TABLE_HEAD
					$last_ipblock = $b_ipblock ;
				}

				$num_listed++ ;
				print <<HTML_TABLE;
      <tr>
        <td><a href="/cgi-bin/cisco/maclog.cgi?location=$location&amp;findipaddr=$ipaddr$url_listdate" target="_blank">$ipaddr</a>&nbsp;</td>
        <td><a href="/cgi-bin/cisco/maclog.cgi?location=$location&amp;findmac=$mac$url_listdate" target="_blank">$mac</a>&nbsp;</td>
        <td>$hostname&nbsp;</td>
        <td align="right">$vlan&nbsp;</td>
        <td>$date $time&nbsp;</td>
        <td>$vendor&nbsp;</td>
      </tr>

HTML_TABLE
			}
		}
	}
}

###############################################################################
# count mac addresses with multiple ipaddr entries
###############################################################################

$num_multi_ipaddr = 0 ;
$num_msum_ipaddr = 0 ;
foreach $key (keys %mac2ipaddr)
{
	my @ipaddrs = split (",", $mac2ipaddr {$key}) ;
	if ($#ipaddrs > 0)
	{
		$num_multi_ipaddr++ ;
		$num_msum_ipaddr++ ;
		$num_msum_ipaddr = $num_msum_ipaddr + $#ipaddrs ;
	}
}

###############################################################################
# display HTML tail
###############################################################################

my $FN_OUI = basename ($OUI) ;
print <<HTML_TAIL;
    </table>

    <h2>$prgnam $location statistics</h2>

    <p>
      $num_ignorearp arp entries removed (ignorearp=$do_ignorearp)<br>
      $num_ignoremac mac entries removed (ignoremac=$do_ignoremac)<br>
      $num_listed mac/ipaddr pairs listed (filter may be applied)<br>
      $num_arp arp entries (with history)<br>
      $num_multi_ipaddr macs have (had) multiple ipaddr ($num_msum_ipaddr)<br>
      $num_dns reverse DNS entries resolved (by either DNS or /etc/hosts)<br>
      $num_oui OUI entries from $FN_OUI<br>
    </p>

    <p>$prgnam $version</p>

  </body>

</html>
HTML_TAIL
