#! /usr/bin/perl
#
###############################################################################
#
# Copyright (C) 2004-2009 by Kostis Netzwerkberatung
# Talstr. 25, D-63322 Roedermark, Germany
# kosta@kostis.de, http://www.kostis.de/
#
# 2009-02-03: V1.40 KK
#     - re-release
#
# You may use this software free of charge at your own risk.
#
###############################################################################
#
# display arplog data in HTML table format resolving hostname
#
###############################################################################

$location = shift ;

require "CiscoTools.pl" ;

###############################################################################

my $version = "1.40 (2009-02-03)" ;
my $prgnam = basename ($0) ;

###############################################################################
# HTML title
###############################################################################

$TITLE="CiscoTools $prgnam $location" ;
$H1="<strong><font color=\"#0000A0\">Cisco</font><font color=\"#0000FF\">Tools</font></strong> $prgnam $location" ;

###############################################################################
#
# main ()
#
###############################################################################

###############################################################################
# read data
###############################################################################

my $num_oui = read_oui () ;	# read OUI data (from IEEE)
my $num_arp = read_arp () ;	# read arplog data (show arp)

###############################################################################
# remove unwanted data
###############################################################################

my $num_ignorearp = 0 ;
if ("$do_ignorearp" eq "yes")
{
        $num_ignorearp = read_ignorearp () ;
}

my $num_ignoremac = 0 ;
if ("$do_ignoremac" eq "yes")
{
        $num_ignoremac = read_ignoremac () ;
}

###############################################################################
# map data
###############################################################################

my $num_arp = map_arp () ;	# map mac to ipaddr and vice versa
my $num_dns = map_dns () ;	# map ipaddr to hostname

###############################################################################
# main (:)
###############################################################################

print <<HTML_HEAD;
Content-type: text/html

<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN">

<html>

  <head>

    <title>$TITLE</title>
    <meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1">
    <meta http-equiv=Content-Language content=en>
    <meta content="CiscoTools" name=description>
    <meta content="Kostis Netzwerkberatung" name=author>
    <meta content="Kostis Netzwerkberatung" name=publisher>
    <meta content="Kostis Netzwerkberatung" name=copyright>
    <meta name="GENERATOR" content="$prgnam $version">
    <link rev="made" href="mailto:webmaster@kostis.de">
    <link href="/cisco/style.css" rel=stylesheet>

  </head>

  <body>

    <h1>$H1</h1>

HTML_HEAD

###############################################################################
# check if user is allowed to view this location
###############################################################################

read_locations () ;

if (not exists ($plocations {$location}))
{
###############################################################################
	print <<HTML_NOPERM;
    <p>Your account $username has no permission to view location $location</p>

    <p>$prgnam $version</p>

  </body>

</html>
HTML_NOPERM
###############################################################################
	exit ;
}

if ("$listdate" ne "")
{
	print <<HTML_LISTDATE;

    <h2>display limited to data $listdate</h2>

HTML_LISTDATE
}

print <<HTML_TABLE_HEAD;
    <table>

      <tr>
        <td class="title">mac&nbsp;</td>
        <td class="title">ipaddr&nbsp;</td>
        <td class="title">hostname&nbsp;</td>
        <td class="title">vlan&nbsp;</td>
        <td class="title">last&nbsp;seen&nbsp;</td>
        <td class="title">vendor&nbsp;</td>
     </tr>

HTML_TABLE_HEAD

###############################################################################
# display data
###############################################################################

foreach my $key (sort keys %arplog)
{ 
	(my $mac, my $ipaddr, my $vlan) = split (",", $key) ;
	my $vendor = mac2vendor ($mac) ;
	my $hostname = ipaddr2hostname ($ipaddr) ;
	my $ldate = $arplog {$key} ;
	$ldate =~ s/_/ / ;
	(my $date, my $time) = split (" ", $ldate) ;

	my $doit = "no" ;
	if ("$listdate" eq "")
	{
		$doit = "yes" ;
	}
	elsif (("$listafer" ne "") && ("$ldate" ge "$listafter"))
	{
		$doit = "yes" ;
	}
	elsif (("$listbefore" ne "") && ("$ldate" lt "$listbefore"))
	{
		$doit = "yes" ;
	}

	if ("$doit" eq "yes")
	{
		print <<HTML_TABLE;
      <tr>
        <td><a href="/cgi-bin/cisco/maclog.cgi?location=$location&amp;findmac=$mac$url_listdate" target="_blank">$mac</a>&nbsp;</td>
        <td><a href="/cgi-bin/cisco/maclog.cgi?location=$location&amp;findipaddr$url_listdate=$ipaddr" target="_blank">$ipaddr</a>&nbsp;</td>
        <td>$hostname&nbsp;</td>
        <td>$vlan&nbsp;</td>
        <td>$date $time&nbsp;</td>
        <td>$vendor&nbsp;</td>
      </tr>

HTML_TABLE
	}
}

###############################################################################
# count mac addresses with multiple ipaddr entries
###############################################################################

my $num_multi_ipaddr = 0 ;
my $num_msum_ipaddr = 0 ;
foreach $key (keys %mac2ipaddr)
{
	my @ipaddr = split (",", $mac2ipaddr {$key}) ;
	if ($#ipaddr > 0)
	{
		$num_multi_ipaddr++ ;
		$num_msum_ipaddr++ ;
		$num_msum_ipaddr = $num_msum_ipaddr + $#ipaddr ;
	}
}

###############################################################################
# display HTML tail
###############################################################################

my $FN_OUI = basename ($OUI) ;
print <<HTML_TAIL;
    </table>

    <h2>$prgnam $location statistics</h2>

    <p>
      $num_ignorearp arp entries removed (ignorearp=$do_ignorearp)<br>
      $num_ignoremac mac entries removed (ignoremac=$do_ignoremac)<br>
      $num_arp arp entries (with history)<br>
      $num_multi_ipaddr macs have (had) multiple ipaddr ($num_msum_ipaddr)<br>
      $num_dns reverse DNS entries resolved (by either DNS or /etc/hosts)<br>
      $num_oui OUI entries from $FN_OUI<br>
    </p>

    <p>$prgnam $version</p>

  </body>

</html>
HTML_TAIL
