#! /usr/bin/perl
#
###############################################################################
#
# Copyright (C) 2004-2009 by Kostis Netzwerkberatung
# Talstr. 25, D-63322 Roedermark, Germany
# kosta@kostis.de, http://www.kostis.de/
#
# 2009-02-03: V1.40 KK
#     - initial re-release
#
# You may use this software free of charge at your own risk.
#
###############################################################################
#
# display potential duplicate ipaddr
#
###############################################################################
#

require "CiscoTools.pl" ;

###############################################################################

my $version="1.40 (2009-02-03)" ;
my $prgnam = basename ($0) ;

###############################################################################
# HTML title
###############################################################################

$TITLE="CiscoTools $prgnam $location" ;
$H1="<strong><font color=\"#0000A0\">Cisco</font><font color=\"#0000FF\">Tools</font></strong> $prgnam $location" ;

###############################################################################
#
# main ()
#
###############################################################################

###############################################################################
# read data
###############################################################################

read_oui () ;
read_arp () ;
read_mac () ;
read_ignoremac () ;

###############################################################################
# map data
###############################################################################

map_arp () ;
map_dns () ;
map_mac () ;

###############################################################################
# print HTML head
###############################################################################

print <<HTML_HEAD;
Content-type: text/html

<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">

<html>

  <head>

    <title>$TITLE</title>
    <meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1">
    <meta http-equiv=Content-Language content=en>
    <meta content="CiscoTools" name=description>
    <meta content="Kostis Netzwerkberatung" name=author>
    <meta content="Kostis Netzwerkberatung" name=publisher>
    <meta content="Kostis Netzwerkberatung" name=copyright>
    <meta name="GENERATOR" content="$prgnam $version">
    <link rev="made" href="mailto:webmaster@kostis.de">
    <link href="/cisco/style.css" rel=stylesheet>

  </head>

  <body>

    <h1>$H1</h1>

HTML_HEAD

###############################################################################
# check if user is allowed to view this location
###############################################################################

read_locations () ;

if (not exists ($plocations {$location}))
{
###############################################################################
	print <<HTML_NOPERM;
    <p>Your account $username has no permission to view location $location</p>

    <p>$prgnam $version</p>

  </body>

</html>
HTML_NOPERM
###############################################################################
	exit ;
}

print <<HTML_TABLE;
    <table border="1">

HTML_TABLE

my $num_duplicate = 0 ;
foreach my $ipaddr (sort keys %ipaddr2macvlan)
{
	my @mactab = split (",", $ipaddr2macvlan {$ipaddr}) ;

	if (scalar (@mactab) > 1)
	{
		$num_duplicate++ ;

print <<HTML_TABLE_HEAD;
      <tr>
        <td class="title">ipaddr&nbsp;</td>
        <td class="title">mac&nbsp;</td>
        <td class="title">hostname&nbsp;</td>
        <td class="title">vlan&nbsp;</td>
        <td class="title">last&nbsp;seen&nbsp;</td>
        <td class="title">vendor&nbsp;</td>
     </tr>
HTML_TABLE_HEAD

		my $hostname = ipaddr2hostname ($ipaddr) ;

		foreach my $macvlan (sort @mactab)
		{
			($mac, $vlan) = split (";", $macvlan) ;
			my $vendor = mac2vendor ($mac) ;
			my $arpkey = "$mac,$ipaddr,$vlan" ;
			my $date = "" ;
			my $time = "" ;
			if (exists ($arplog {$arpkey}))
			{
				($date, $time) = split ("_", $arplog {$arpkey}) ;
			}

			print <<HTML_TABLE_LINE;

      <tr>
        <td><a href="/cgi-bin/cisco/maclog.cgi?location=$location&amp;findipaddr=$ipaddr" target="_blank">$ipaddr</a>&nbsp;</td>
        <td><a href="/cgi-bin/cisco/maclog.cgi?location=$location&amp;findmac=$mac" target="_blank">$mac</a>&nbsp;</td>
        <td>$hostname&nbsp;</td>
        <td>$vlan&nbsp;</td>
        <td>$date $time&nbsp;</td>
        <td>$vendor&nbsp;</td>
      </tr>

HTML_TABLE_LINE
		}
	}
}

###############################################################################
# print HTML tail
###############################################################################

my $FN_IGNOREMAC = basename ($IGNOREMAC) ;
print <<HTML_TAIL;
    </table>

    <h2>$prgnam $location statistics</h2>
	
    <p>$num_duplicate potential duplicate ipaddr detected.</p>

    <p>
      An ipaddr is listed as duplicate if more than one mac
      (excluding ignoremac) has been associated with an ipaddr.
    </p>

    <p>
      duplicate ipaddr may have a number of causes:

      <ul>
        <li>ethernet interface replaced</libb>
        <li>dhcp server with small pool and/or short lease time</libb>
        <li>cluster type service</li>
        <li>fault tolerant protocol (e. g. Cisco hsrp)</li>
        <li>real duplicate ipaddr (e. g. by manual configuration - avoid this!)</li>
      </ul>

      $prgnam ignores the mac addresses listed in $FN_IGNOREMAC
    </p>

    <p>$prgnam $version</p>

  </body>

</html>
HTML_TAIL
