#! /usr/bin/perl
#
###############################################################################
#
# Copyright (C) 2008-2009 by Kostis Netzwerkberatung
# Talstr. 25, D-63322 Roedermark, Germany
# kosta@kostis.de, http://www.kostis.de/
#
# 2009-02-03: V1.40 KK
#     - initial release
#
# You may use this software free of charge at your own risk.
#
###############################################################################
#
# display portmaps.txt in HTML table format
#
###############################################################################
#

require "CiscoTools.pl" ;

###############################################################################

my $version="1.40 (2009-02-03)" ;
my $prgnam = basename ($0) ;

###############################################################################
# HTML title and settings
###############################################################################

$TITLE="CiscoTools $prgnam $location" ;
$H1="<strong><font color=\"#0000A0\">Cisco</font><font color=\"#0000FF\">Tools</font></strong> $prgnam $location" ;

###############################################################################
#
# main ()
#
###############################################################################

print <<HTML_HEAD;
Content-type: text/html

<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">

<html>

  <head>

    <title>$TITLE</title>
    <meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1">
    <meta http-equiv=Content-Language content=en>
    <meta content="CiscoTools" name=description>
    <meta content="Kostis Netzwerkberatung" name=author>
    <meta content="Kostis Netzwerkberatung" name=publisher>
    <meta content="Kostis Netzwerkberatung" name=copyright>
    <meta name="GENERATOR" content="$prgnam $version">
    <link rev="made" href="mailto:kosta@kostis.de">
    <link href="/cisco/style.css" rel=stylesheet>

  </head>

  <body>

    <h1>$H1</h1>

HTML_HEAD

###############################################################################
# check if user is allowed to view this location
###############################################################################

read_locations () ;

if (not exists ($plocations {$location}))
{
###############################################################################
	print <<HTML_NOPERM;
    <p>Your account $username has no permission to view location $location</p>

    <p>$prgnam $version</p>

  </body>

</html>
HTML_NOPERM
###############################################################################
	exit ;
}

###############################################################################
# if listdate is given say so
###############################################################################

if ("$listdate" ne "")
{
	print <<HTML_LISTDATE;
    <h2>Report limited to active ipaddr $listdate</h2>

HTML_LISTDATE
}
else
{
	print <<HTML_NO_LISTDATE;
    <p>
      You may limit the report by adding &quot;&amp;listafter=isodate&quot;
      or &quot;&amp;listbefore=isodate&quot; to the URL
      (e. g. &amp;listafter=2008-09-08_00:00:00).
    </p>

HTML_NO_LISTDATE
}

###############################################################################
# display table header
###############################################################################

print <<HTML_TABLE_HEAD;
    <table>

      <tr>
        <td class="title">ipblock/24</td>
        <td class="title"># of ipaddr</td>
        <td class="title">description</td>
      </tr>
HTML_TABLE_HEAD

###############################################################################
# read arp data and ipblocks descriptions
###############################################################################

my $num_arp = read_arp () ;
my $num_ipblocks = read_ipblocks () ;

###############################################################################
# walk through all arplog entries and make a list with ipaddr key
###############################################################################

my %ipaddrs = () ;
my $ipblock = "" ;
foreach my $key (sort keys %arplog)
{
	#######################################################################
	# determine last seen date and remove "_"
	#######################################################################

	my $ldate = $arplog {$key} ;
	$ldate =~ s/_/ / ;

	#######################################################################
	# shall we add this to our display list?
	#######################################################################

	my $doit = "no" ;
	if ("$listdate" eq "")
	{
		$doit = "yes" ;
	}
	elsif (("$listafter" ne "") && ("$ldate" ge "$listafter"))
	{
		$doit = "yes" ;
	}
	elsif (("$listbefore" ne "") && ("$ldate" lt "$listbefore"))
	{
		$doit = "yes" ;
	}


	if ("$doit" eq "yes")
	{
		###############################################################
		# count ipaddr in ipblock
		###############################################################

		(my $mac, my $ipaddr, my $vlan) = split (",", $key) ;
		$ipblock = ipblock ($ipaddr) ;
		if (not exists ($ipaddrs {$ipblock}))
		{
			$ipaddrs {$ipblock} = 0 ;
		}
		++$ipaddrs {$ipblock} ;
	}
}

###############################################################################
# walk through list of ipblocks
###############################################################################

my $num_subnets = 0 ;
my $num_ipaddrs = 0 ;
foreach $ipblock (sort { ($hc{$a} ||= inet_aton($a)) cmp ($hc{$b} ||= inet_aton($b)) } keys %ipaddrs)
{
	my $num_ipaddr = $ipaddrs {$ipblock} ;
	$num_ipaddrs += $num_ipaddr ;
	my $description = "" ;
	if (exists ($ipblocks {$ipblock}))
	{
		$description = $ipblocks {$ipblock} ;
	}

print <<HTML_TABLE;

      <tr>
        <td><a href="/cgi-bin/cisco/arplog-ip.cgi?location=$location&amp;findipblock=$ipblock$url_listdate" target="_blank">$ipblock</a>&nbsp;</td>
        <td align="right">$num_ipaddr&nbsp;</td>
        <td>$description&nbsp;</td>
      </tr>
HTML_TABLE

	$num_subnets++ ;
}

my $num_diff = $num_arp - $num_ipaddrs ;
my $FN_IPBLOCKS = basename ($IPBLOCKS) ;
print <<HTML_TAIL;

    </table>

    <h2>$prgnam $location statistics</h2>

    <p>
      Read $num_arp mac/ipaddr pairs<br>
      Read $num_ipblocks ipblocks from $FN_IPBLOCKS<br>
      Found $num_subnets ipblocks/24 and a total of $num_ipaddrs ipaddr.<br>
      $num_diff mac/ipaddr pairs discarted by filter.<br>
      Actual subnet masks are unknown to this script.
    </p>

    <p>$prgnam $version</p>

  </body>

</html>
HTML_TAIL
