#! /usr/bin/perl
#
###############################################################################
#
# Copyright (C) 2004-2009 by Kostis Netzwerkberatung
# Talstr. 25, D-63322 Roedermark, Germany
# kosta@kostis.de, http://www.kostis.de/
#
# 2009-02-02: V1.40 KK
#     - initial re-release
#
# You may use this software free of charge at your own risk.
#
###############################################################################
#
# display maclog data in CSV table format resolving ipaddr and hostname
#
###############################################################################

###############################################################################
# global data hashes for statistical data
###############################################################################

require "CiscoTools.pl" ;

###############################################################################

my $version = "1.40 (2009-02-03)" ;
my $prgnam = basename ($0) ;

###############################################################################
#
# main ()
#
###############################################################################

###############################################################################
# check if user is allowed to view this location
###############################################################################

read_locations () ;

if (not exists ($plocations {$location}))
{
###############################################################################
	print <<TXT_NOPERM;
Content-type: text/plain

Your account $username has no permission to view location $location

$prgnam $version
TXT_NOPERM
###############################################################################
	exit ;
}

###############################################################################
# display TEXT header
###############################################################################

print <<TEXT_HEAD;
Content-type: text/plain

mac;switch;interface;ipaddr;hostname;vlan;date;time;vendor
TEXT_HEAD

###############################################################################
# read data
###############################################################################

read_oui () ;				# read OUI data (from IEEE)
read_arp () ;				# read arplog data (show arp)
read_mac () ;				# read maclog data (show mac-address-table)
read_portmaps () ;			# read portmap data

###############################################################################
# map data
###############################################################################

map_arp () ;				# map mac to ipaddr and vice versa
map_dns () ;				# map ipaddr to hostname
map_mac () ;				# map mac to interface and vice versa

###############################################################################
# this loop goes through all arplog data
###############################################################################

foreach my $key (sort keys %maclog)
{ 
	#######################################################################
	# this list is sorted by mac,switch,interface,vlan
	#######################################################################

	(my $mac, my $switch, my $interface, $vlan) = split (",", $key) ;
	my $vendor = mac2vendor ($mac) ;
	$isodate = $maclog {$key} ;
	($date, $time) = split ("_", $isodate) ;

	#######################################################################
	# map mac back to ipaddrs
	#######################################################################

	my $maclist = "n/a" ;		# make sure there at least one entry
	if (exists ($mac2ipaddr {$mac}))
	{
		$maclist = $mac2ipaddr {$mac} ;
	}

	#######################################################################
	# walk through all ipaddr associated with mac
	#######################################################################

	@iplist = () ;
	@iplist = split (",", $maclist) ;

	#######################################################################
	# the following loop works only if there is at least one entry.
	# since macs may not have a corresponding ipaddr, we use "n/a".
	#######################################################################

	foreach my $ipaddr (sort @iplist) 
	{
		###############################################################
		# assume no valid vlan
		###############################################################

		my $ivlan = "n/a" ;

		###############################################################
		# determine vlan if available
		###############################################################

		if (exists ($ipaddr2vlan {$ipaddr}))
		{
			$ivlan = $ipaddr2vlan {$ipaddr} ;
		}

		###############################################################
		# if vlan is given but does not match skip it
		###############################################################

		if (("$ivlan" ne "n/a") && ("$vlan" ne "$ivlan"))
		{
			next ;
		}

		###############################################################
		# figure out hostname for ipaddr (if available)
		###############################################################

		$hostname = ipaddr2hostname ($ipaddr) ;

		print <<CSV_LINE;
$mac;$switch;$interface;$ipaddr;$hostname;$vlan;$date;$time;$vendor
CSV_LINE
	}
}
