#! /usr/bin/perl
#
###############################################################################
#
# Copyright (C) 2004-2009 by Kostis Netzwerkberatung
# Talstr. 25, D-63322 Roedermark, Germany
# kosta@kostis.de, http://www.kostis.de/
#
# 2009-02-03: V1.40 KK
#     - initial re-release
#
# You may use this software free of charge at your own risk.
#
###############################################################################
#
# display maclog data in HTML table format resolving ipaddr and hostname
#
###############################################################################

###############################################################################
# global data hashes for statistical data
###############################################################################

%switches = () ;
%interfaces = () ;
%Fas = () ;
%Gis = () ;
%Pos = () ;
%Tes = () ;

require "CiscoTools.pl" ;

###############################################################################

my $version = "1.40 (2009-02-03)" ;
my $prgnam = basename ($0) ;

###############################################################################
# HTML title
###############################################################################

$TITLE="CiscoTools $prgnam $location" ;
$H1="<strong><font color=\"#0000A0\">Cisco</font><font color=\"#0000FF\">Tools</font></strong> $prgnam $location" ;

###############################################################################
# count interfaces depending on prefix (Te, Gi, Fa)
###############################################################################

sub CountInterface ($$)
{
	my $switch = shift ;
	my $interface = shift ;

	#######################################################################
	# add switch to list of switches
	#######################################################################

	if (not exists ($switches {$switch}))
	{
		$switches {$switch} = 1 ;
	}

	#######################################################################
	# create hashes if necessary
	#######################################################################

	if (not exists ($Fas {$switch}))
	{
		$Fas {$switch} = 0 ;
	}
	if (not exists ($Gis {$switch}))
	{
		$Gis {$switch} = 0 ;
	}
	if (not exists ($Pos {$switch}))
	{
		$Pos {$switch} = 0 ;
	}
	if (not exists ($Tes {$switch}))
	{
		$Tes {$switch} = 0 ;
	}

	#######################################################################
	# check if interface was already counted
	#######################################################################

	my $key = $switch . "," . $interface ;
	if (exists ($interfaces {$key}))
	{
		return (0) ;
	}

	#######################################################################
	# count interfaces by type
	#######################################################################

	my $int = substr ($interface, 0, 2) ;
	if ("$int" eq "Fa")
	{
		++$Fas {$switch} ;
	}
	elsif ("$int" eq "Gi")
	{
		++$Gis {$switch} ;
	}
	elsif ("$int" eq "Po")
	{
		++$Pos {$switch} ;
	}
	elsif ("$int" eq "Te")
	{
		++$Tes {$switch} ;
	}

	#######################################################################
	# remember this interface was counted already
	#######################################################################

	$interfaces {$key} = 1 ;

	return (1) ;
}

###############################################################################
# add up number of interfaces (for types Te,Gi,Fa)
###############################################################################

sub AddUpInterfaces ()
{
	my $switch ;

	my $Fa = 0 ;
	foreach $switch (keys %Fas)
	{
		$Fa += $Fas {$switch} ;
	}

	my $Gi = 0 ;
	foreach $switch (keys %Gis)
	{
		$Gi += $Gis {$switch} ;
	}

	my $Po = 0 ;
	foreach $switch (keys %Pos)
	{
		$Po += $Pos {$switch} ;
	}

	my $Te = 0 ;
	foreach $switch (keys %Tes)
	{
		$Te += $Tes {$switch} ;
	}

	return ("$Fa,$Gi,$Te,$Po") ;
}

###############################################################################
# add portmaps (they use up interfaces, too)
###############################################################################

sub AddPortmaps ()
{
	foreach my $map (sort keys %portmaps)
	{ 
		(my $switch, my $interface) = split (",", $map) ;
		CountInterface ($switch, $interface) ;
	}
}

###############################################################################
#
# main (:)
#
###############################################################################

###############################################################################
# display HTML header
###############################################################################

print <<HTML_HEAD;
Content-type: text/html

<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">

<html>

  <head>

    <title>$TITLE</title>
    <meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1">
    <meta http-equiv=Content-Language content=en>
    <meta content="CiscoTools" name=description>
    <meta content="Kostis Netzwerkberatung" name=author>
    <meta content="Kostis Netzwerkberatung" name=publisher>
    <meta content="Kostis Netzwerkberatung" name=copyright>
    <meta name="GENERATOR" content="$prgnam $version">
    <link rev="made" href="mailto:webmaster@kostis.de">
    <link href="/cisco/style.css" rel=stylesheet>

  </head>

  <body>

    <h1>$H1</h1>
HTML_HEAD

###############################################################################
# check if user is allowed to view this location
###############################################################################

read_locations () ;

if (not exists ($plocations {$location}))
{
###############################################################################
	print <<HTML_NOPERM;
    <p>Your account $username has no permission to view location $location</p>

    <p>$prgnam $version</p>

  </body>

</html>
HTML_NOPERM
###############################################################################
	exit ;
}

my $findthis = "" ;
if ("$findipaddr" ne "")
{
	$findthis = "ipaddr " . $findipaddr ;
}
elsif ("$findmac" ne "")
{
	$findthis = "mac " . $findmac ;
}
elsif ("$findswitch" ne "")
{
	$findthis = "switch " . $findswitch ;
}
elsif ("$findvendor" ne "")
{
	$findthis = "vendor " . $findvendor ;
}

if ("$findinterface" ne "")
{
	$findthis = $findthis . " interface " . $findinterface ;
}

if ("$findthis" ne "")
{
	print <<HTML_FINDMAC;
    <h2>Searching for $findthis</h2>
HTML_FINDMAC
}

if ("$do_ignoremac" eq "yes")
{
	print <<HTML_IGNOREMAC;

    <h2>ignoring selected mac addresses</h2>
HTML_IGNOREMAC
}

if ("$listdate" ne "")
{
	print <<HTML_LISTDATE;

    <h2>display limited to data $listdate</h2>
HTML_LISTDATE
}

print <<HTML_TABLE;

    <table>

HTML_TABLE

###############################################################################
# read data
###############################################################################

my $num_oui = read_oui () ;		# read OUI data (from IEEE)
my $num_arp = read_arp () ;		# read arplog data
my $num_mac = read_mac () ;		# read maclog data
my $num_portmaps = read_portmaps () ;	# read portmaps data

my $num_ignorearp = 0 ;
if ("$do_ignorearp" eq "yes")
{
	$num_ignorearp = read_ignorearp () ;
}

my $num_ignoremac = 0 ;
if ("$do_ignoremac" eq "yes")
{
	$num_ignoremac = read_ignoremac () ;
}

###############################################################################
# map data
###############################################################################

my $num_map_arp = map_arp () ;		# map mac to ipaddr and vice versa
my $num_map_dns = map_dns () ;		# map ipaddr to hostname
my $num_map_mac = map_mac () ;		# map mac to interface and vice versa

my $num_mac = keys %mac2interface ;	# determine number of macs
my $num_portmaps = keys %portmaps ;	# determine number of portmaps
my $num_ipaddr = keys %ipaddr2mac ;	# determine number of ipaddr

###############################################################################
# this loop goes through interfaces
###############################################################################

my $last_switch = "" ;

foreach my $map (sort keys %interface2vlan)
{ 
	#######################################################################
	# this list is sorted by switch,interface
	#######################################################################

	(my $switch, my $interface) = split (",", $map) ;

	#######################################################################
	# count interface for switch
	#######################################################################

	CountInterface ($switch, $interface) ;

	#######################################################################
	# an interface may see multiple vlans
	#######################################################################

	my @vlantab = split (",", $interface2vlan {"$map"}) ;

	#######################################################################
	# walk through vlans on switch interface
	#######################################################################

	foreach my $vlan (sort @vlantab)
	{
		my $mkey = $map . "," . $vlan ;

		if (not exists ($interface2mac {$mkey}))
		{
			next ;
		}

		my @mactab = split (",", $interface2mac {$mkey}) ;

		###############################################################
		# walk through macs on switch interface and vlan
		###############################################################

		foreach my $mac (sort @mactab)
		{
			#######################################################
			# determine vendor
			#######################################################

			my $vendor = mac2vendor ($mac) ;

			#######################################################
			# check if maclog entry exists
			#######################################################

			my $key = "$mac,$mkey" ;

			if (not exists ($maclog {$key}))
			{
				next ;
			}

			#######################################################
			# yes, get date and go on
			#######################################################

			$isodate = $maclog {$key} ;
			$isodate =~ s/_/ /;
			($date, $time) = split (" ", $isodate) ;

			#######################################################
			# map mac back to ipaddrs
			#######################################################

			my $maclist = "n/a" ;
			if (exists ($mac2ipaddr {$mac}))
			{
				$maclist = $mac2ipaddr {$mac} ;
			}

			#######################################################
			# walk through all ipaddr associated with mac
			#######################################################

			@iplist = () ;
			@iplist = split (",", $maclist) ;
			foreach my $ipaddr (sort @iplist) 
			{
				my $ivlan = "n/a" ;
				if (exists ($ipaddr2vlan {$ipaddr}))
				{
					$ivlan = $ipaddr2vlan {$ipaddr} ;
				}

				if (("$ivlan" ne "n/a") && ("$vlan" ne "$ivlan"))
				{
					next ;
				}

				$hostname = ipaddr2hostname ($ipaddr) ;

				my $doit = "no" ;		# assume we do not list
				if ("$findthis" eq "")
				{
					$doit = "yes" ;
				}
				elsif (("$findmac" ne "") && ("$findmac" eq "$mac"))
				{
					$doit = "yes" ;
				}
				elsif (("$findipaddr" ne "") && ("$findipaddr" eq "$ipaddr"))
				{
					$doit = "yes" ;
				}
				elsif (("$findswitch" ne "") && ("$findswitch" eq "$switch"))
				{
					$doit = "yes" ;
					if (("$findinterface" ne "") && ("$findinterface" ne "$interface"))
					{
						$doit = "no" ;
					}
				}
				elsif (("$findvendor" ne "") && ("$findvendor" eq substr ($mac, 0, 8)))
				{
					$doit = "yes" ;
				}

				###############################################
				# date restrictions
				###############################################

				if (("$listafter" ne "") && ("$isodate" lt "$listafter"))
				{
					$doit = "no" ;
				}
				elsif (("$listbefore" ne "") && ("$isodate" gt "$listbefore"))
				{
					$doit = "no" ;
				}

				if ("$doit" eq "yes")
				{
					if ("$last_switch" ne "$switch")
					{
						print <<HTML_TABLE_HEAD;
      <tr>
        <td class="title">switch&nbsp;</td>
        <td class="title">interface&nbsp;</td>
        <td class="title">mac&nbsp;</td>
        <td class="title">ipaddr&nbsp;</td>
        <td class="title">hostname&nbsp;</td>
        <td class="title">vlan&nbsp;</td>
        <td class="title">last&nbsp;seen&nbsp;</td>
        <td class="title">vendor&nbsp;</td>
      </tr>

HTML_TABLE_HEAD
						$last_switch = $switch ;
					}

					my $im = "" ;
					if ("$do_ignoremac" eq "yes")
					{
						$im = "&amp;ignoremac=$do_ignoremac" ;
					}

					print <<HTML_TABLE_MAIN;
      <tr>
        <td><a href="/cgi-bin/cisco/maclog.cgi?location=$location&amp;findswitch=$switch$url_listdate$im" target="_blank">$switch</a>&nbsp;</td>
        <td><a href="/cgi-bin/cisco/maclog.cgi?location=$location&amp;findswitch=$switch&amp;findinterface=$interface$url_listdate$im" target="_blank">$interface</a>&nbsp;</td>
        <td><a href="/cgi-bin/cisco/maclog.cgi?location=$location&amp;findmac=$mac$url_listdate$im" target="_blank">$mac</a>&nbsp;</td>
        <td><a href="/cgi-bin/cisco/maclog.cgi?location=$location&amp;findipaddr=$ipaddr$url_listdate$im" target="_blank">$ipaddr</a>&nbsp;</td>
        <td>$hostname&nbsp;</td>
        <td>$vlan&nbsp;</td>
        <td>$date $time&nbsp;</td>
        <td>$vendor&nbsp;</td>
      </tr>

HTML_TABLE_MAIN
				}
			}
		}
	}
}

print <<HTML_TABLE_TAIL;
    </table>
HTML_TABLE_TAIL

###############################################################################
# display statistics only for full listings
###############################################################################

if ("$findthis" eq "")
{

	#######################################################################
	# count vendors and count occurances
	#######################################################################

	%vendormac = () ;
	foreach $mac (sort keys %mac2interface)
	{
		my $vendor = substr ($mac, 0, 8) ;
		if (not exists ($vendormac {$vendor}))
		{
			$vendormac {$vendor} = 0 ;
		}
		$vendormac {$vendor}++ ;
	}

	$num_vendor = keys %vendormac ;
	$num_switch = keys %switches ;

	#######################################################################
	# display statistics
	#######################################################################

	print <<HTML_STAT;

    <h2>$prgnam $location statistics</h2>

    <p>
HTML_STAT

	(my $Fa, my $Gi, my $Te, my $Po) = split (",", AddUpInterfaces ()) ;

	print <<HTML_STAT_EXCL;
     $Po Po(rt-channel), $Te Te(nGigabit), $Gi Gi(gabit), $Fa Fa(st) Ethernet interfaces in use (excl. portmaps)<br>
HTML_STAT_EXCL

	AddPortmaps () ;
	(my $Fa, my $Gi, my $Te, my $Po) = split (",", AddUpInterfaces ()) ;

	print <<HTML_STAT_INCL;
     $Po Po(rt-channel), $Te Te(nGigabit), $Gi Gi(gabit), $Fa Fa(st) Ethernet interfaces in use (incl. portmaps)<br>
    </p>
HTML_STAT_INCL

	print <<HTML_SWITCH_LIST_HEAD;

    <h3>$num_switch switches listed</h3>

    <table border="1">

      <tr>
        <td class="title">switch&nbsp;</td>
        <td class="title">Po&nbsp;</td>
        <td class="title">Te&nbsp;</td>
        <td class="title">Gi&nbsp;</td>
        <td class="title">Fa&nbsp;</td>
      </tr>

HTML_SWITCH_LIST_HEAD

	foreach $switch (sort keys %switches)
	{
		print <<HTML_SWITCH_LIST;
      <tr>
        <td>$switch</td>
        <td align="right">$Pos{$switch}&nbsp;</td>
        <td align="right">$Tes{$switch}&nbsp;</td>
        <td align="right">$Gis{$switch}&nbsp;</td>
        <td align="right">$Fas{$switch}&nbsp;</td>
      </tr>

HTML_SWITCH_LIST
	}

	$num_vlans = count_vlans () ;

print <<HTML_SWITCH_VLAN;
    </table>

    <h3>$num_vlans vlans listed</h3>

    <table border="1">
      <tr>
        <td class="title">vlan</td>
        <td class="title"># of macs</td>
      </tr>

HTML_SWITCH_VLAN

	foreach $vlan (sort {$a <=> $b} keys %countvlans)
	{
		print <<HTML_VLAN_LIST;
      <tr>
        <td align="right">$vlan</td>
        <td align="right">$countvlans{$vlan}</td>
      </tr>

HTML_VLAN_LIST
	}

	my $FN_OUI = basename ($OUI) ;
	my $FN_PORTMAPS = basename ($PORTMAPS) ;
	my $FN_IGNOREMAC = basename ($IGNOREMAC) ;
	print <<HTML_STAT_SUMMARY;
    </table>

    <p>
      $num_ignorearp arp entries removed (ignorearp=$do_ignorearp)<br>
      $num_ignoremac mac entries removed (ignoremac=$do_ignoremac)<br>
      $num_mac different mac found<br>
      $num_arp arp entries found<br>
      $num_ipaddr different ipaddr found<br>
      $num_map_dns reverse DNS entries resolved<br>
      $num_portmaps entries read from $FN_PORTMAPS<br>
      $num_oui OUI entries read from $FN_OUI<br>
    </p>

    <h3>$num_vendor OUIs (vendor ids) detected</h3>

    <table border="1">

      <tr>
        <td class="title">OUI&nbsp;</td>
        <td class="title">vendor&nbsp;</td>
        <td class="title"># of macs&nbsp;</td>
      </tr>

HTML_STAT_SUMMARY

	foreach $vendor (sort keys %vendormac)
	{
		my $vendorname = "" ;
		if (exists ($oui {$vendor}))
		{
			$vendorname = $oui {$vendor} ;
		}
		print <<HTML_MAC_VENDOR;
      <tr>
        <td><a href="/cgi-bin/cisco/maclog.cgi?location=$location&amp;findvendor=$vendor$url_listdate" target="_blank">$vendor</a>&nbsp;</td>
        <td>$vendorname&nbsp;</td>
        <td align="right">$vendormac{$vendor}&nbsp;</td>
      </tr>

HTML_MAC_VENDOR
	}

	print <<HTML_STAT_END ;
    </table>
HTML_STAT_END

}

print <<HTML_TAIL ;

    <p>$prgnam $version</p>

  </body>

</html>
HTML_TAIL
