#! /usr/bin/perl
#
###############################################################################
#
# Copyright (C) 2004-2009 by Kostis Netzwerkberatung
# Talstr. 25, D-63322 Roedermark, Germany
# kosta@kostis.de, http://www.kostis.de/
#
# 2009-02-03: V1.40 KK
#     - initial re-release
#
# You may use this software free of charge at your own risk.
#
###############################################################################
#
# display muliple mac addresses behind an interface
#
###############################################################################
#

require "CiscoTools.pl" ;

###############################################################################

my $version="1.40 (2009-02-03)" ;
my $prgnam = basename ($0) ;

###############################################################################
# HTML title
###############################################################################

$TITLE="CiscoTools $prgnam $location" ;
$H1="<strong><font color=\"#0000A0\">Cisco</font><font color=\"#0000FF\">Tools</font></strong> $prgnam $location" ;

###############################################################################
#
# main ()
#
###############################################################################

###############################################################################
# read data
###############################################################################

read_oui () ;
read_arp () ;
read_mac () ;
read_ignoremac () ;

###############################################################################
# map data
###############################################################################

map_arp () ;
map_dns () ;
map_mac () ;

###############################################################################
# print HTML head
###############################################################################

print <<HTML_HEAD;
Content-type: text/html

<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">

<html>

  <head>

    <title>$TITLE</title>
    <meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1">
    <meta http-equiv=Content-Language content=en>
    <meta content="CiscoTools" name=description>
    <meta content="Kostis Netzwerkberatung" name=author>
    <meta content="Kostis Netzwerkberatung" name=publisher>
    <meta content="Kostis Netzwerkberatung" name=copyright>
    <meta name="GENERATOR" content="$prgnam $version">
    <link rev="made" href="mailto:webmaster@kostis.de">
    <link href="/cisco/style.css" rel=stylesheet>

  </head>

  <body>

    <h1>$H1</h1>

HTML_HEAD

print <<HTML_TABLE;
    <table border="1">

HTML_TABLE

###############################################################################
# check if user is allowed to view this location
###############################################################################

read_locations () ;

if (not exists ($plocations {$location}))
{
###############################################################################
	print <<HTML_NOPERM;
    <p>Your account $username has no permission to view location $location</p>

    <p>$prgnam $version</p>

  </body>

</html>
HTML_NOPERM
###############################################################################
	exit ;
}

my $num_multiple = 0 ;
foreach my $map (sort keys %interface2mac)
{
	(my $switch, my $interface, my $vlan) = split (",", $map) ;
	my @mactab = split (",", $interface2mac {$map}) ;
	if ($#mactab > 0)
	{
		print <<HTML_TABLE_HEAD;
      <tr>
        <td class="title">switch&nbsp;</td>
        <td class="title">interface&nbsp;</td>
        <td class="title">mac&nbsp;</td>
        <td class="title">ipaddr&nbsp;</td>
        <td class="title">hostname&nbsp;</td>
        <td class="title">vlan&nbsp;</td>
        <td class="title">last&nbsp;seen&nbsp;</td>
        <td class="title">vendor&nbsp;</td>
     </tr>

HTML_TABLE_HEAD

		$num_multiple++ ;
		foreach my $mac (sort @mactab)
		{
			my $vendor = mac2vendor ($mac) ;
			my $key = "$mac,$map" ;

			my $isodate = $maclog {$key} ;
			(my $date, my $time) = split ("_", $isodate) ;

			my $ipaddrs = $mac2ipaddr {$mac} ;
			if ("$ipaddrs" ne "")
			{
				@iplist = split (",", $ipaddrs) ;
			}
			else
			{
				@iplist = ("") ;
			} 

	                foreach my $ipaddr (sort @iplist)
        	        {
				my $hostname = ipaddr2hostname ($ipaddr) ;
				print <<HTML_TABLE_LINE;
      <tr>
        <td><a href="/cgi-bin/cisco/maclog.cgi?location=$location&amp;findswitch=$switch" target="_blank">$switch</a>&nbsp;</td>
        <td><a href="/cgi-bin/cisco/maclog.cgi?location=$location&amp;findswitch=$switch&amp;findinterface=$interface$url_listdate" target="_blank">$interface</a>&nbsp;</td>
        <td><a href="/cgi-bin/cisco/maclog.cgi?location=$location&amp;findmac=$mac" target="_blank">$mac</a>&nbsp;</td>
        <td><a href="/cgi-bin/cisco/maclog.cgi?location=$location&amp;findipaddr=$ipaddr" target="_blank">$ipaddr</a>&nbsp;</td>
        <td>$hostname&nbsp;</td>
        <td>$vlan&nbsp;</td>
        <td>$date $time&nbsp;</td>
        <td>$vendor&nbsp;</td>
      </tr>

HTML_TABLE_LINE
			}
		}
	}
}

###############################################################################
# print HTML tail
###############################################################################

print <<HTML_TAIL;
    </table>

    <h2>$prgnam $location statistics</h2>
	
    <p>$num_multiple interfaces have seen multiple mac addresses</p>

    <p>
      If multiple Vlans are configured on a switch interface
      the Vlan listed here may not be entirely correct...
    </p>

    <p>
      An interface is listed as having multiple mac addresses
      if more than one mac address (excluding ignoremac) was
      seen there since the last cleanup of maclog.
    </p>

    <p>
      Multiple mac addresses may have a number of causes:

      <ul>
        <li>undocumented switch/hub connection (check portmaps.txt)</li>
        <li>unauthorized switch/hub (check portmaps.txt)</li>
        <li>unmanaged switch/hub (no cheap away around that)</li>
        <li>multiple hosts on a physical interface (e. g. VMware)</li>
        <li>fault tolerant devices/applications</li>
      </ul>

      $prgnam ignores mac addresses listed in ignoremac.txt
    </p>

    <p>$prgnam $version</p>

  </body>

</html>
HTML_TAIL
