#! /usr/bin/perl
#
###############################################################################
#
# Copyright (C) 2004-2009 by Kostis Netzwerkberatung
# Talstr. 25, D-63322 Roedermark, Germany
# kosta@kostis.de, http://www.kostis.de/
#
# 2009-02-03: V1.40 KK
#     - initial re-release
#
# You may use this software free of charge at your own risk.
#
###############################################################################
#
# display portmaps.txt in HTML table format
#
###############################################################################
#

require "CiscoTools.pl" ;

###############################################################################

my $version="1.40 (2009-02-03)" ;
my $prgnam = basename ($0) ;

###############################################################################
# HTML title and settings
###############################################################################

$TITLE="CiscoTools $prgnam $location" ;
$H1="<strong><font color=\"#0000A0\">Cisco</font><font color=\"#0000FF\">Tools</font></strong> $prgnam $location" ;

###############################################################################
#
# sub chk_portmaps ()
#
# chk portmaps consistency
#
###############################################################################

sub chk_portmaps ()
{
	my $count = 0 ;
	foreach my $key (sort keys %portmaps)
	{
		my $val = $portmaps {$key} ;

		if (not exists ($portmaps {$val}))
		{
			print ("      ERROR: entry for key $key with val $val has no corresponding return entry<br>\n") ;
			$count++ ;
		}
		else
		{
			if ("$portmaps{$val}" ne "$key")
			{
				print ("      ERROR: inconsistent data for $key and $val<br>\n") ;
				$count++ ;
			}
		}
	}

	return ($count) ;
}

###############################################################################
#
# main ()
#
###############################################################################

my $num_portmaps = read_portmaps () ;

print <<HTML_HEAD;
Content-type: text/html

<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN"

<html>

  <head>

    <title>$TITLE</title>
    <meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1">
    <meta http-equiv=Content-Language content=en>
    <meta content="CiscoTools" name=description>
    <meta content="Kostis Netzwerkberatung" name=author>
    <meta content="Kostis Netzwerkberatung" name=publisher>
    <meta content="Kostis Netzwerkberatung" name=copyright>
    <meta name="GENERATOR" content="$prgnam $version">
    <link rev="made" href="mailto:kosta@kostis.de">
    <link href="/cisco/style.css" rel=stylesheet>

  </head>

  <body>

    <h1>$H1</h1>

HTML_HEAD

###############################################################################
# check if user is allowed to view this location
###############################################################################

read_locations () ;

if (not exists ($plocations {$location}))
{
###############################################################################
	print <<HTML_NOPERM;
    <p>Your account $username has no permission to view location $location</p>

    <p>$prgnam $version</p>

  </body>

</html>
HTML_NOPERM
###############################################################################
	exit ;
}

print <<HTML_TABLE_HEAD;
    <table>

      <tr>
        <td class="title">switch</td>
        <td class="title">interface</td>
        <td class="title">a_switch</td>
        <td class="title">a_interface</td>
      </tr>

HTML_TABLE_HEAD

my %switches = () ;
foreach my $key (sort keys %portmaps)
{
	(my $switch, my $interface) = split (",", $key) ;
	(my $a_switch, my $a_interface) = split (",", $portmaps {$key}) ;
	if (not exists ($switches {$switch}))
	{
		$switches {$switch} = 1 ;
	}
	if (not exists ($switches {$a_switch}))
	{
		$switches {$a_switch} = 1 ;
	}
	print <<HTML_TABLE_LINE;
      <tr>
        <td><a href="/cgi-bin/cisco/maclog.cgi?location=$location&amp;findswitch=$switch$url_listdate" target="_blank">$switch</a>&nbsp;</td>
        <td>$interface&nbsp;</td>
        <td><a href="/cgi-bin/cisco/maclog.cgi?location=$location&amp;findswitch=$a_switch$url_listdate" target="_blank">$a_switch</a>&nbsp;</td>
        <td>$a_interface&nbsp;</td>
      </tr>

HTML_TABLE_LINE
}

print <<HTML_TABLE_TAIL;
    </table>

    <p>
HTML_TABLE_TAIL

my $FN_PORTMAPS = basename ($PORTMAPS) ;
my $num_switches = keys %switches ;
my $num_errors = chk_portmaps () ;

print <<HTML_TAIL;
    </p>

    <h2>$prgnam $location statistics</h2>

    <p>
      file $FN_PORTMAPS contains $num_portmaps entries, $num_switches switches and $num_errors errors.
    </p>

    <p>$prgnam $version</p>

  </body>

</html>
HTML_TAIL
