#! /usr/bin/perl
#
###############################################################################
#
# Copyright (C) 2004-2009 by Kostis Netzwerkberatung
# Talstr. 25, D-63322 Roedermark, Germany
# kosta@kostis.de, http://www.kostis.de/
#
# 2009-02-03: V1.40 KK
#     - initial release
#
# You may use this software free of charge at your own risk.
#
###############################################################################
#
# display roaming mac adresses in HTML table format
#
###############################################################################
#

require "CiscoTools.pl" ;

###############################################################################

my $version="1.40 (2009-02-03)" ;
my $prgnam = basename ($0) ;

###############################################################################
# HTML title
###############################################################################

$TITLE="CiscoTools $prgnam $location" ;
$H1="<strong><font color=\"#0000A0\">Cisco</font><font color=\"#0000FF\">Tools</font></strong> $prgnam $location" ;

###############################################################################
#
# main ()
#
###############################################################################

###############################################################################
# read data
###############################################################################

read_oui () ;
read_arp () ;
read_mac () ;
read_ignoremac () ;

###############################################################################
# map data
###############################################################################

map_arp () ;
map_dns () ;
map_mac () ;

###############################################################################
# display HTML head
###############################################################################

print <<HTML_HEAD ;
Content-type: text/html

<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN">

<html>

  <head>

    <title>$TITLE</title>
    <meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1">
    <meta http-equiv=Content-Language content=en>
    <meta content="CiscoTools" name=description>
    <meta content="Kostis Netzwerkberatung" name=author>
    <meta content="Kostis Netzwerkberatung" name=publisher>
    <meta content="Kostis Netzwerkberatung" name=copyright>
    <meta name="GENERATOR" content="$prgnam $version">
    <link rev="made" href="mailto:webmaster@kostis.de">
    <link href="/cisco/style.css" rel=stylesheet>

  </head>

  <body>

    <h1>$H1</h1>

HTML_HEAD

###############################################################################
# check if user is allowed to view this location
###############################################################################

read_locations () ;

if (not exists ($plocations {$location}))
{
###############################################################################
	print <<HTML_NOPERM;
    <p>Your account $username has no permission to view location $location</p>

    <p>$prgnam $version</p>

  </body>

</html>
HTML_NOPERM
###############################################################################
	exit ;
}

print <<HTML_TABLE_HEAD ;
    <table border="1">

HTML_TABLE_HEAD

###############################################################################
# map mac to switch interface (without vlan)
###############################################################################

my %roamingmac = () ;
foreach my $mac (sort keys %mac2interface)
{
	my @interfaces = split (";", $mac2interface {$mac}) ;
	foreach my $swint (sort @interfaces)
	{
		(my $switch, my $interface, my $vlan) = split (",", $swint) ;
		my $val = "$switch,$interface" ;
		if (not exists ($roamingmac {$mac}))
		{
			$roamingmac {$mac} = $val ;
		}
		else
		{
			$roamingmac {$mac} = add_element ($roamingmac {$mac}, $val, ";") ;
		}
	}
}

###############################################################################
# walk through mac on switch interfaces
###############################################################################

my $num_roaming = 0 ;
foreach my $mac (sort keys %roamingmac)
{
	my $vendor = mac2vendor ($mac) ;
	my @interfaces = split (";", $roamingmac {$mac}) ;

	#######################################################################
	# check if more than one interface is mapped to given mac (roaming)
	#######################################################################

	if (scalar (@interfaces) > 1)
	{
		$num_roaming++ ;

		my $ipaddr = "n/a" ;
		if (exists ($mac2ipaddr {$mac}))
		{
			#######################################################
			# take first ipaddr
			#######################################################

			@ipaddrtab = split (",", $mac2ipaddr {$mac}) ;
			$ipaddr = $ipaddrtab [0] ;
		}
		my $hostname = ipaddr2hostname ($ipaddr) ;

		print <<HTML_TABLE_HEAD;
      <tr>
        <td class="title">mac&nbsp;</td>
        <td class="title">switch&nbsp;</td>
        <td class="title">interface&nbsp;</td>
        <td class="title">ipaddr&nbsp;</td>
        <td class="title">hostname&nbsp;</td>
        <td class="title">vendor&nbsp;</td>
      </tr>

HTML_TABLE_HEAD

		foreach my $map (sort @interfaces)
		{
			(my $switch, my $interface) = split (",", $map) ;
			print <<HTML_TABLE_LINE;
      <tr>
        <td><a href="/cgi-bin/cisco/maclog.cgi?location=$location&amp;findmac=$mac" target="_blank">$mac</a>&nbsp;</td>
        <td><a href="/cgi-bin/cisco/maclog.cgi?location=$location&amp;findswitch=$switch" target="_blank">$switch</a>&nbsp;</td>
        <td><a href="/cgi-bin/cisco/maclog.cgi?location=$location&amp;findswitch=$switch&amp;findinterface=$interface$url_listdate" target="_blank">$interface</a>&nbsp;</td>
        <td><a href="/cgi-bin/cisco/maclog.cgi?location=$location&amp;findipaddr=$ipaddr" target="_blank">$ipaddr</a>&nbsp;</td>
        <td>$hostname&nbsp;</td>
        <td>$vendor&nbsp;</td>
      </tr>

HTML_TABLE_LINE
		}
	}
}

###############################################################################
# print HTML tail
###############################################################################

my $FN_IGNOREMAC = basename ($IGNOREMAC) ;
print <<HTML_TAIL;
    </table>

    <h2>$prgnam $location statistics</h2>

    <p>$num_roaming roaming mac addresses detected</p>

    <p>
      A mac address is listed as a roaming mac address if it was
      seen on more than one switch interface.<br>
      This can generally be caused by machines being moved
      between switches, roaming devices (e.g. Notebooks)
      or by fault tolerant devices (e.g. routers/firewalls).
    </p>

    <p>ipaddr listed is ONE of the ipaddr associated with the given mac - there may be others</p>

    <p>$prgnam ignores mac addresses listed in $FN_IGNOREMAC</p>

    <p>$prgnam $version</p>

  </body>

</html>
HTML_TAIL
